/*
VisImage

Copyright (C) 2004 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.*;
import java.util.ResourceBundle;

/**
 * Affichage d'une image avec les coordonnes x,y
 * + outils: point, ligne, cercle
 */
public class VisImage extends Applet implements ItemListener, ActionListener, TextListener {

    static ResourceBundle rb  = ResourceBundle.getBundle("LocalStrings");
    int POINT = 1;
    int LIGNE = 2;
    int CERCLE = 3;
    
    ImageCanvas ic;
    Label labcoords;
    TextArea historique;
    String[] thisto;
    int nhisto = 4;
    Checkbox chkpoint;
    Checkbox chkligne;
    Checkbox chkcercle;
    CheckboxGroup cbg;
    Button boutonEffacer;
    TextField tfunite;
    
    public void init() {
        setLayout(new BorderLayout());
        String nomImage = getParameter("image");
        Image im = getImage(getCodeBase(), nomImage);
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(im, 0);
        try {
            tracker.waitForAll();
        } catch (InterruptedException ex) {
            System.err.println("VisImage: InterruptedException: " + ex.getMessage());
        }
        ic = new ImageCanvas(im, this);
        add(ic, BorderLayout.CENTER);
        Panel toolpane = new Panel();
        Panel checkpane = new Panel(new GridLayout(5, 1));
        checkpane.add(new Label(rb.getString("outils")));
        cbg = new CheckboxGroup();
        chkpoint = new Checkbox(rb.getString("point"), cbg, true);
        chkpoint.addItemListener(this);
        checkpane.add(chkpoint);
        chkligne = new Checkbox(rb.getString("ligne"), cbg, false);
        checkpane.add(chkligne);
        chkligne.addItemListener(this);
        chkcercle = new Checkbox(rb.getString("cercle"), cbg, false);
        checkpane.add(chkcercle);
        chkcercle.addItemListener(this);
        boutonEffacer = new Button(rb.getString("effacer"));
        boutonEffacer.setActionCommand("effacer");
        boutonEffacer.addActionListener(this);
        checkpane.add(boutonEffacer);
        ic.setOutil(POINT);
        toolpane.add(checkpane);
        add(toolpane, BorderLayout.WEST);
        Panel baspane = new Panel(new BorderLayout());
        Panel coordunitpane = new Panel(new BorderLayout());
        labcoords = new Label(" ");
        coordunitpane.add(labcoords, BorderLayout.CENTER);
        Panel unitpane = new Panel();
        unitpane.add(new Label(rb.getString("rapport")));
        tfunite = new TextField("1", 5);
        tfunite.addTextListener(this);
        unitpane.add(tfunite);
        coordunitpane.add(unitpane, BorderLayout.EAST);
        baspane.add(coordunitpane, BorderLayout.NORTH);
        historique = new TextArea("", nhisto, 50, TextArea.SCROLLBARS_NONE);
		historique.setEditable(false);
        thisto = new String[nhisto];
        for (int i=0; i<nhisto; i++)
            thisto[i] = "";
        baspane.add(historique, BorderLayout.CENTER);
        add(baspane, BorderLayout.SOUTH);
        validate();
        ic.requestFocus();
    }
    
    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();
        if ("effacer".equals(cmd))
            effacer();
    }
    
    public void effacer() {
        ic.effacer();
    }
    
    public void ajouterHisto(String s) {
        for (int i=0; i<nhisto-1; i++)
            thisto[i] = thisto[i+1];
        thisto[nhisto-1] = s;
        afficherHisto();
    }
    
    public void modifierHisto(String s) {
        thisto[nhisto-1] = s;
        afficherHisto();
    }
    
    public void afficherHisto() {
        String s = null;
        for (int i=0; i<nhisto; i++)
            if (s == null)
                s = thisto[i];
            else
                s += "\n" + thisto[i];
        historique.setText(s);
    }
    
    public void itemStateChanged(ItemEvent e) {
        Object sel = e.getItemSelectable();
        if (sel == chkpoint)
            ic.setOutil(POINT);
        else if (sel == chkligne)
            ic.setOutil(LIGNE);
        else if (sel == chkcercle)
            ic.setOutil(CERCLE);
    }
    
    public void textValueChanged(TextEvent e) {
        double rapport;
        try {
            rapport = (Double.valueOf(tfunite.getText())).doubleValue();
        } catch (NumberFormatException ex) {
            rapport = 0;
        }
        ic.setRapportUnite(rapport);
    }
    
    class ImageCanvas extends Canvas implements MouseMotionListener, MouseListener, KeyListener {
        Image img, buffer;
        Dimension size;
        int w, h;
        int outil;
        Point p1 = null, p2 = null;
        double rapport;
        Graphics buffg;
        
        // on suppose ici que l'image est dj charge
        public ImageCanvas(Image image, Applet app) {
            
            if (image == null) {
                System.err.println("VisImage.ImageCanvas: erreur: image == null");
                return;
            }
    
            w = image.getWidth(null);
            h = image.getHeight(null);
            
            if (w == -1 || h == -1) {
                System.err.println("VisImage.ImageCanvas: erreur: image non charge");
                w = 100;
                h = 100;
            }
            
            size = new Dimension(w,h);
            
            img = image;
            buffer = app.createImage(w, h);
            buffg = buffer.getGraphics();
            buffg.drawImage(img, 0, 0, this);
            buffg.drawRect(0, 0, w - 1, h - 1);
            
            rapport = 1;
            
            setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
            addMouseMotionListener(this);
            addMouseListener(this);
            addKeyListener(this);
        }
    
        public Dimension getPreferredSize() {
            return getMinimumSize();
        }
    
        public Dimension getMinimumSize() {
            return size;
        }
    
        public void paint(Graphics g) {
            g.drawImage(buffer, 0, 0, this);
        }
        
        public void update(Graphics g) {
            paint(g);
        }
        
        public void mouseMoved(MouseEvent e) {
            String slab = rb.getString("coordonnes") + " " + e.getX() + ", " + e.getY();
            labcoords.setText(slab);
        }
        
        public void mouseDragged(MouseEvent e) {
            if (outil == LIGNE && p1 != null) {
                boolean nouveau = (p2 == null);
                if (!nouveau)
                    dessinerLigne();
                p2 = e.getPoint();
                dessinerLigne();
                if (nouveau)
                    ajouterHisto(texteHisto());
                else
                    modifierHisto(texteHisto());
            } else if (outil == CERCLE && p1 != null) {
                boolean nouveau = (p2 == null);
                if (!nouveau)
                    dessinerCercle();
                p2 = e.getPoint();
                dessinerCercle();
                if (nouveau)
                    ajouterHisto(texteHisto());
                else
                    modifierHisto(texteHisto());
            }
            repaint();
        }
        
        public void mouseClicked(MouseEvent e) {
            if (outil == POINT) {
                //if (p1 != null)
                //    dessinerPoint();
                p1 = e.getPoint();
                dessinerPoint();
                repaint();
                ajouterHisto(texteHisto());
            }
        }
        
        public void mouseEntered(MouseEvent e) {
        }
        
        public void mouseExited(MouseEvent e) {
        }
        
        public void mousePressed(MouseEvent e) {
            if (outil == LIGNE) {
                if (p2 != null) {
                    //dessinerLigne();
                    //repaint();
                    p2 = null;
                }
                p1 = e.getPoint();
            } else if (outil == CERCLE) {
                if (p2 != null) {
                    //dessinerCercle();
                    //repaint();
                    p2 = null;
                }
                p1 = e.getPoint();
            }
        }
        
        public void mouseReleased(MouseEvent e) {
            if (outil == LIGNE) {
                if (p2 != null)
                    dessinerLigne();
                p2 = e.getPoint();
                dessinerLigne();
                repaint();
            }
        }
        
        public void keyPressed(KeyEvent e) {
            switch (e.getKeyCode()) {
                case KeyEvent.VK_LEFT: deplacer(-1, 0); break;
                case KeyEvent.VK_RIGHT: deplacer(1, 0); break;
                case KeyEvent.VK_UP: deplacer(0, -1); break;
                case KeyEvent.VK_DOWN: deplacer(0, 1); break;
                case KeyEvent.VK_ADD: agrandir(1); break;
                case KeyEvent.VK_SUBTRACT: agrandir(-1); break;
                default: ;
            }
        }
        
        public void keyReleased(KeyEvent e) {
        }
        
        public void keyTyped(KeyEvent e) {
        }
        
        public void setRapportUnite(double rapport) {
            this.rapport = rapport;
            modifierHisto(texteHisto());
        }
        
        public void effacer() {
            buffg.drawImage(img, 0, 0, this);
            buffg.drawRect(0, 0, w - 1, h - 1);
            repaint();
        }
        
        public void deplacer(int dx, int dy) {
            dessinerObjet();
            if (p1 != null) {
                p1.x += dx;
                p1.y += dy;
            }
            if (p2 != null) {
                p2.x += dx;
                p2.y += dy;
            }
            dessinerObjet();
            repaint();
            if (outil == POINT && p1 != null)
                modifierHisto(texteHisto());
            else if (outil == CERCLE && p1 != null)
                modifierHisto(texteHisto());
        }
        
        public void agrandir(int dr) {
            if (outil == CERCLE && p1 != null && p2 != null) {
                dessinerCercle();
                if (p2.x > p1.x)
                    p2.x += dr;
                else
                    p2.x -= dr;
                if (p2.y > p1.y)
                    p2.y += dr;
                else
                    p2.y -= dr;
                dessinerCercle();
                repaint();
                modifierHisto(texteHisto());
            }
        }
        
        public String texteHisto() {
            if (outil == POINT && p1 != null) {
                return(rb.getString("point_en") + " " + p1.x + ", " + p1.y);
            } else if (outil == LIGNE && p2 != null) {
                double l = Math.sqrt(Math.pow(p2.x - p1.x, 2) +
                    Math.pow(p2.y - p1.y, 2));
                l = Math.rint(l*10)/10;
                return(rb.getString("ligne_de_longueur") + " " + l + " (" + joliFormattage(l*rapport, 3) + " u)");
            } else if (outil == CERCLE && p2 != null) {
                int rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
                return(rb.getString("cercle_de_centre") + " ("+p1.x+","+p1.y+") " + rb.getString("et_de_rayon") + " " + rayon +
                    " (" + joliFormattage(rayon*rapport, 3) + " u)");
            }
            return(null);
        }
        
        public void setOutil(int o) {
            /*
            if (outil == POINT && p1 != null)
                dessinerPoint();
            else if (outil == LIGNE && p2 != null)
                dessinerLigne();
            else if (outil == CERCLE && p2 != null)
                dessinerCercle();
            */
            outil = o;
            p1 = null;
            p2 = null;
            repaint();
        }
        
        public void dessinerObjet() {
            if (outil == POINT && p1 != null)
                dessinerPoint();
            if (outil == LIGNE && p1 != null && p2 != null)
                dessinerLigne();
            if (outil == CERCLE && p1 != null && p2 != null)
                dessinerCercle();
        }
        
        public void dessinerPoint() {
            buffg.setXORMode(Color.white);
            buffg.drawLine(p1.x - 5, p1.y, p1.x + 5, p1.y);
            buffg.drawLine(p1.x, p1.y - 5, p1.x, p1.y + 5);
            buffg.setPaintMode();
        }
        
        public void dessinerLigne() {
            buffg.setXORMode(Color.white);
            buffg.drawLine(p1.x, p1.y, p2.x, p2.y);
            buffg.setPaintMode();
        }
        
        public void dessinerCercle() {
            buffg.setXORMode(Color.white);
            int rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
            buffg.drawOval(p1.x - rayon, p1.y - rayon, rayon*2, rayon*2);
            buffg.setPaintMode();
        }
    }
    
    public static String joliFormattage(double v, int precision) {
        int limitesci = 3; // dcimales avant de passer en mode sci
        
        if (v == 0)
            return("0");
        String sj = ""+v;
        if ("Infinity".equals(sj) || "-Infinity".equals(sj) || "NaN".equals(sj))
            return(""+v);
        String signe;
        if (v >= 0)
            signe = "";
        else {
            v = -v;
            signe = "-";
        }
        int exposant = (int)Math.floor(Math.log(v)/Math.log(10));
        double exparr = Math.pow(10, precision);
        double mantisse = v / Math.pow(10, exposant);
        mantisse = Math.rint(mantisse*exparr)/exparr;
        if (mantisse >= 10) {
            exposant += 1;
            mantisse = v / Math.pow(10, exposant);
            mantisse = Math.rint(mantisse*exparr)/exparr;
        }
        String sv = "";
        int chiffre = (int)Math.floor(mantisse);
        sv += chiffre + ".";
        for (int i=0; i<precision-1; i++) {
            mantisse = (mantisse - chiffre)*10;
            double exparr2 = Math.pow(10, precision-i);
            mantisse = Math.rint(mantisse*exparr)/exparr;
            chiffre = (int)Math.floor(mantisse);
            sv += chiffre;
        }
        mantisse = (mantisse - chiffre)*10;
        chiffre = (int)Math.round(mantisse);
        sv += chiffre;
        char dernier = sv.charAt(sv.length()-1);
        while (dernier == '0') {
            sv = sv.substring(0, sv.length()-1);
            dernier = sv.charAt(sv.length()-1);
        }
        if (exposant != 0) {
            if (exposant > 0 && exposant < limitesci) {
                String sd = sv.substring(0,1);
                if (sv.length()-2 < exposant) {
                    // transformation de 3.2E2 en 320
                    if (sv.length() > 2)
                        sd += sv.substring(2,sv.length());
                    for (int i=0; i<exposant-(sv.length()-2); i++)
                        sd += '0';
                } else {
                    // transformation de 3.245E2 en 324.5
                    if (sv.length() < exposant+2)
                        sd += sv.substring(2);
                    else
                        sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
                }
                sv = sd;
            } else if (exposant < 0 && exposant > -limitesci) {
                // transformation de 3.2E-2 en 0.032
                String sd = "0.";
                for (int i=0; i<-exposant-1; i++)
                    sd += '0';
                sv = sd + sv.charAt(0) + sv.substring(2);
            } else
                sv += "E" + exposant;
        }
        if (sv.charAt(sv.length()-1) == '.')
            sv = sv.substring(0, sv.length()-1);
        
        sv = signe + sv;
        
        return(sv);
    }

}
