/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var image;
var canvas;
var context;
var canvastrace;
var ctxtrace;
var outil; // 1=point, 2=ligne, 3=cercle
var dernierObjet;
var p1, p2;
var nhisto = 4; // nombre de lignes maximum dans l'historique
var thisto;
var drag;
var rapport;

function lireParametres() {
    var parametres = new Array();
    var searchString = window.location.search.substring(1);
    var params = searchString.split("&");
    
    for (var i=0; i<params.length; i++) {
        var nomval = params[i].split("=");
        // remarque: les paramètres de l'URL doivent être encodés en UNICODE
        parametres[nomval[0]] = decodeURIComponent(nomval[1]);
    }
    return(parametres);
}

function init() {
    var parametres = lireParametres();
    var chemin_image = parametres['image'];
    var chemin_fichiers = parametres['chemin_fichiers'];
    if (chemin_fichiers != null)
        chemin_image = chemin_fichiers + chemin_image;
    image = new Image();
    image.onload = premierAffichage;
    image.onerror = function(err) { alert("Erreur au chargement de l'image " + chemin_image) };
    image.src = chemin_image;
    outil = 1;
    var bligne = document.getElementById('bligne');
    if (bligne.checked)
        outil = 2;
    var bcercle = document.getElementById('bcercle');
    if (bcercle.checked)
        outil = 3;
    dernierObjet = 0;
    p1 = null;
    p2 = null;
    thisto = new Array();
    for (var i=0; i<nhisto; i++)
        thisto[i] = '';
    drag = false;
    var inputrapport = document.getElementById('rapport');
    rapport = inputrapport.value;
    if (inputrapport.addEventListener)
        inputrapport.addEventListener('keyup', modifRapport, false);
    else
        inputrapport.attachEvent('onkeyup', modifRapport);
}

function premierAffichage() {
    var divimage = document.getElementById('divimage');
    canvas = document.createElement('canvas');
    canvas.setAttribute('width', image.width);
    canvas.setAttribute('height', image.height);
    divimage.appendChild(canvas);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        G_vmlCanvasManager.initElement(canvas);
    }
    
    if (!canvas.getContext) {
        divimage.innerHTML = '<p>Votre navigateur ne gère pas HTML5.</p>';
        return;
    }
    
    context = canvas.getContext('2d');
    // les bitmaps offscreen sont impossibles avec excanvas et flashcanvas
    // pour que ça marche avec IE7 et IE8, on utilise un autre élément canvas
    // pour tracer les lignes et les cercles
    canvastrace = document.createElement('canvas');
    canvastrace.setAttribute('width', image.width);
    canvastrace.setAttribute('height', image.height);
    // setAttribute ne marche pas pour style avec IE...
    canvastrace.style.position = 'absolute';
    canvastrace.style.left = '0px';
    canvastrace.style.top = '0px';
    divimage.appendChild(canvastrace);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        canvastrace = G_vmlCanvasManager.initElement(canvastrace);
    }
    ctxtrace = canvastrace.getContext('2d');
    
    if (canvastrace.addEventListener) {
        canvastrace.addEventListener('mousedown', mousedown, false);
        canvastrace.addEventListener('mousemove', mousemove, false);
        canvastrace.addEventListener('mouseup', mouseup, false);
        canvastrace.addEventListener('click', clic, false);
        canvastrace.addEventListener('mouseout', mouseout, false);
    } else {
        canvastrace.attachEvent('onmousedown', mousedown);
        canvastrace.attachEvent('onmousemove', mousemove);
        canvastrace.attachEvent('onmouseup', mouseup);
        canvastrace.attachEvent('onclick', clic);
        canvastrace.attachEvent('onmouseout', mouseout);
    }
    if ('ontouchstart' in document) {
        canvastrace.addEventListener('touchstart', mousedown, false);
        canvastrace.addEventListener('touchmove', mousemove, false);
        canvastrace.addEventListener('touchend', mouseup, false);
    }
    
    if (document.addEventListener) {
        document.addEventListener('keydown', keydown, false);
        document.addEventListener('keypress', keypress, false);
    } else if (document.attachEvent) {
        document.attachEvent('onkeydown', keydown);
        document.attachEvent('onkeypress', keypress);
    }
    
    context.drawImage(image, 0, 0);
}

function choixOutil(numero) {
    outil = numero;
}

function effacer() {
    effacerTrace();
    context.drawImage(image, 0, 0);
    dernierObjet = 0;
    for (var i=0; i<nhisto; i++)
        thisto[i] = '';
    afficherHisto();
}

function effacerTrace() {
    ctxtrace.clearRect(0, 0, image.width, image.height);
}

function ajouterHisto(s) {
    for (var i=0; i<nhisto-1; i++)
        thisto[i] = thisto[i+1];
    thisto[nhisto-1] = s;
    afficherHisto();
}

function modifierHisto(s) {
    thisto[nhisto-1] = s;
    afficherHisto();
}

function afficherHisto() {
    var s = null;
    for (var i=0; i<nhisto; i++)
        if (s == null)
            s = thisto[i];
        else
            s += '\n' + thisto[i];
    var historique = document.getElementById('historique');
    historique.value = s;
    historique.scrollTop = historique.scrollHeight;
}

function texteHisto() {
    if (outil == 1 && p1 != null) {
        return('point en' + ' ' + p1.x + ', ' + p1.y);
    } else if (outil == 2 && p2 != null) {
        var l = Math.sqrt(Math.pow(p2.x - p1.x, 2) + Math.pow(p2.y - p1.y, 2));
        l = Math.round(l*10)/10;
        return('ligne de longueur' + ' ' + l + ' (' + joliFormattage(l*rapport, 3) + ' u)');
    } else if (outil == 3 && p2 != null) {
        var rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
        return('cercle de centre' + ' ('+p1.x+','+p1.y+') ' + 'et de rayon' + ' ' + rayon +
            ' (' + joliFormattage(rayon*rapport, 3) + ' u)');
    }
    return(null);
}

function modifRapport(e) {
    var inputrapport = document.getElementById('rapport');
    rapport = inputrapport.value;
}

function coordonnees(e) {
    // résultat décalé de 1px avec excanvas :(
    var x, y;
    if (e.pageX || e.pageY) {
      x = e.pageX;
      y = e.pageY;
    } else {
      x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
      y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    }
    var totalOffsetX = 0;
    var totalOffsetY = 0;
    var currentElement = canvastrace;
    do {
        totalOffsetX += currentElement.offsetLeft;
        totalOffsetY += currentElement.offsetTop;
    } while (currentElement = currentElement.offsetParent)
    x -= totalOffsetX;
    y -= totalOffsetY;
    return( { x : x, y : y } );
}

function mousedown(e) {
    if (dernierObjet != 0) {
        var voutil = outil;
        outil = dernierObjet;
        dessinerObjet(context);
        outil = voutil;
    }
    effacerTrace();
    dernierObjet = outil;
    if (outil == 2 || outil == 3) {
        var pt = coordonnees(e);
        if (p2 != null)
            p2 = null;
        p1 = pt;
        drag = true;
        if (e.preventDefault)
            e.preventDefault();
        else
            e.returnValue = false;
    }
}

function mousemove(e) {
    var pt = coordonnees(e);
    var spancoord = document.getElementById('coordonnees');
    spancoord.innerHTML = 'coordonnées: ' + pt.x + ', ' + pt.y;
    if (drag) {
        if (outil == 2 && p1 != null) {
            var nouveau = (p2 == null);
            if (!nouveau)
                effacerTrace();
            p2 = pt;
            dessinerLigne(ctxtrace);
            if (nouveau)
                ajouterHisto(texteHisto());
            else
                modifierHisto(texteHisto());
        } else if (outil == 3 && p1 != null) {
            var nouveau = (p2 == null);
            if (!nouveau)
                effacerTrace();
            p2 = pt;
            dessinerCercle(ctxtrace);
            if (nouveau)
                ajouterHisto(texteHisto());
            else
                modifierHisto(texteHisto());
        }
    }
}

function mouseup(e) {
    drag = false;
}

function clic(e) {
    var pt = coordonnees(e);
    var spancoord = document.getElementById('coordonnees');
    spancoord.innerHTML = 'coordonnées: ' + pt.x + ', ' + pt.y;
    if (outil == 1) {
        p1 = pt;
        dessinerPoint(ctxtrace);
        ajouterHisto(texteHisto());
    }
}

function mouseout(e) {
    if (drag)
        mouseup(e);
}

function keydown(e) {
    switch (e.keyCode) {
        case 37: deplacer(-1, 0); break; // left
        case 39: deplacer(1, 0); break; // right
        case 38: deplacer(0, -1); break; // up
        case 40: deplacer(0, 1); break; // down
        default: ;
    }
    if (e.keyCode >= 37 && e.keyCode <= 40) {
        if (e.preventDefault)
            e.preventDefault();
        else
            e.returnValue = false;
    }
}

function keypress(e) {
    if (e.ctrlKey || e.metaKey)
        return;
    var code;
    if (e.which == null)
        code = e.keyCode;    // old IE
    else if (e.which != 0 && e.charCode != 0)
        code = e.which;	  // All others
    else
        return; // special key
    var c = String.fromCharCode(code);
    switch (c) {
        case '+': agrandir(1); break;
        case '-': agrandir(-1); break;
        default: ;
    }
}

function deplacer(dx, dy) {
    effacerTrace();
    if (p1 != null) {
        p1.x += dx;
        p1.y += dy;
    }
    if (p2 != null) {
        p2.x += dx;
        p2.y += dy;
    }
    dessinerObjet(ctxtrace);
    if (outil == 1 && p1 != null)
        modifierHisto(texteHisto());
    else if (outil == 3 && p1 != null)
        modifierHisto(texteHisto());
}

function agrandir(dr) {
    if (outil == 3 && p1 != null && p2 != null) {
        effacerTrace();
        if (p2.x > p1.x)
            p2.x += dr;
        else
            p2.x -= dr;
        if (p2.y > p1.y)
            p2.y += dr;
        else
            p2.y -= dr;
        dessinerCercle(ctxtrace);
        modifierHisto(texteHisto());
    }
}

function dessinerObjet(ctx) {
    if (outil == 1 && p1 != null)
        dessinerPoint(ctx);
    else if (outil == 2 && p1 != null && p2 != null)
        dessinerLigne(ctx);
    else if (outil == 3 && p1 != null && p2 != null)
        dessinerCercle(ctx);
}

function dessinerPoint(ctx) {
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.moveTo(p1.x - 5 + 0.5, p1.y + 0.5);
    ctx.lineTo(p1.x + 5 + 0.5, p1.y + 0.5);
    ctx.moveTo(p1.x + 0.5, p1.y - 5 + 0.5);
    ctx.lineTo(p1.x + 0.5, p1.y + 5 + 0.5);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.moveTo(p1.x - 0.5 - 5, p1.y - 0.5);
    ctx.lineTo(p1.x - 0.5 + 5, p1.y - 0.5);
    ctx.moveTo(p1.x - 0.5, p1.y - 5 - 0.5);
    ctx.lineTo(p1.x - 0.5, p1.y + 5 - 0.5);
    ctx.stroke();
}

function dessinerLigne(ctx) {
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.moveTo(p1.x + 0.5, p1.y + 0.5);
    ctx.lineTo(p2.x + 0.5, p2.y + 0.5);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.moveTo(p1.x - 0.5, p1.y - 0.5);
    ctx.lineTo(p2.x - 0.5, p2.y - 0.5);
    ctx.stroke();
}

function dessinerCercle(ctx) {
    var rayon = Math.max(Math.abs(p2.x - p1.x), Math.abs(p2.y - p1.y));
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.arc(p1.x + 0.5, p1.y + 0.5, rayon, 0, 2*Math.PI, false);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.arc(p1.x - 0.5, p1.y - 0.5, rayon, 0, 2*Math.PI, false);
    ctx.stroke();
}

// fonction issue du Calcotron
function joliFormattage(v, precision) {
    var limitesci = 3; // décimales avant de passer en mode sci
    
    if (v == 0)
        return('0');
    var sj = ''+v;
    if ('Infinity' === sj || '-Infinity' === sj || 'NaN' === sj)
        return(''+v);
    var signe;
    if (v >= 0)
        signe = '';
    else {
        v = -v;
        signe = '-';
    }
    var exposant = Math.floor(Math.log(v)/Math.log(10));
    var exparr = Math.pow(10, precision);
    var mantisse = v / Math.pow(10, exposant);
    mantisse = Math.round(mantisse*exparr)/exparr;
    if (mantisse >= 10) {
        exposant += 1;
        mantisse = v / Math.pow(10, exposant);
        mantisse = Math.round(mantisse*exparr)/exparr;
    }
    var sv = '';
    var chiffre = Math.floor(mantisse);
    sv += chiffre + '.';
    for (var i=0; i<precision-1; i++) {
        mantisse = (mantisse - chiffre)*10;
        var exparr2 = Math.pow(10, precision-i);
        mantisse = (Math.round(mantisse*exparr))/exparr;
        chiffre = Math.floor(mantisse);
        sv += chiffre;
    }
    mantisse = (mantisse - chiffre)*10;
    chiffre = Math.round(mantisse);
    sv += chiffre;
    var dernier = sv.charAt(sv.length-1);
    while (dernier == '0') {
        sv = sv.substring(0, sv.length-1);
        dernier = sv.charAt(sv.length-1);
    }
    if (exposant != 0) {
        if (exposant > 0 && exposant < limitesci) {
            var sd = sv.substring(0,1);
            if (sv.length-2 < exposant) {
                // transformation de 3.2E2 en 320
                if (sv.length > 2)
                    sd += sv.substring(2,sv.length);
                for (var i=0; i<exposant-(sv.length-2); i++)
                    sd += '0';
            } else {
                // transformation de 3.245E2 en 324.5
                if (sv.length < exposant+2)
                    sd += sv.substring(2);
                else
                    sd += sv.substring(2,exposant+2) + '.' + sv.substring(exposant+2);
            }
            sv = sd;
        } else if (exposant < 0 && exposant > -limitesci) {
            // transformation de 3.2E-2 en 0.032
            var sd = '0.';
            for (var i=0; i<-exposant-1; i++)
                sd += '0';
            sv = sd + sv.charAt(0) + sv.substring(2);
        } else
            sv += 'E' + exposant;
    }
    if (sv.charAt(sv.length-1) == '.')
        sv = sv.substring(0, sv.length-1);
    
    sv = signe + sv;
    
    return(sv);
}


